<?php
/**
 * phpNSTA
 *
 * This file is part of the phpNSTA
 *
 * @license http://www.gnu.de/documents/gpl-2.0.de.html
 * @copyright 2013 - 2016
 */

/**
 * Vereinfacht die Ausgabe von farbigen Texten auf der Konsole
 *
 * Mit hilfe der CLI Klasse, können farbigen auf der Linux Konsole ausgegeben werden.
 *
 * @property array $colors    Array mit allen unterstüzten Farben
 *
 * @uses Config - Statische Klasse um die Konfigurationsdatei zu lesen
 * @package phpNSTA
 * @author Daniel Ziegler <daniel.ziegler@it-novum.com>
 */
class CLI{
	
	/**
	 * Function: __construct
	 *
	 * Erstellt das Object CLI und füllt die benötigten Variablen
	 *
	 * @return void
	 * @since 1.0
	 *
	 */
	function __construct(){
		require_once "static.class.config.php";
		$this->colors = array();
		$this->colors['start'] = "\033[";
		$this->colors['end'] = "\033[0m";
		$this->colors['red'] = "0;31m";
		$this->colors['light_red'] = "1;31m";
		$this->colors['purple'] = "0;35m";
		$this->colors['light_purple'] = "1;35m";
		$this->colors['blue'] = "0;34m";
		$this->colors['light_blue'] = "1;34m";
		$this->colors['green'] = "0;32m";
		$this->colors['light_green'] = "1;32m";
	}

	/**
	 * Function: echoWelcome
	 *
	 * Gibt die Willkommensnachricht des phpNSTA aus
	 *
	 * @return void
	 * @since 1.0
	 *
	 */
	function echoWelcome(){
		echo $this->startColor('purple')."
				|-------------------------------------------------------------------|
				|                      _           _   _ ____ _____  _              |
				|                _ __ | |__  _ __ | \ | / ___|_   _|/ \             |
				|               | '_ \| '_ \| '_ \|  \| \___ \ | | / _ \            |
				|               | |_) | | | | |_) | |\  |___) || |/ ___ \           |
				|               | .__/|_| |_| .__/|_| \_|____/ |_/_/   \_\          |
				|               |_|         |_|                                     |
				|                                                                   |
				| * phpNSTA ".$this->_return(Config::val('MAIN', 'version'), 6)."                                                  |
				|                                                                   |
				| * Developed by Daniel Ziegler 2012-".date('Y')." - it-novum GmbH           |
				| * www.it-novum.com                                                |
				| * www.open-itcockpit.org                                          |
				|-------------------------------------------------------------------|
				\n".$this->endColor();
	}

	/**
	 * Function: startColor
	 *
	 * Färbt einen übergebenen Text in belibige Farben für die CLI
	 *
	 * @param string $color  Die Farbe mit der der Text eingefärbt werden soll
	 * @return string        Der übergebene Text mit Farbcode für die CLI
	 * @since 1.0
	 *
	 */
	function startColor($color){
		if(isset($this->colors[$color])){
			return $this->colors['start'].$this->colors[$color];
		}else{
			return "";
		}
	}

	/**
	 * Function: endColor
	 *
	 * Beendet das einfärben der CLI und setzt alles auf den Standard zurück
	 *
	 * @return void
	 * @since 1.0
	 *
	 */
	function endColor(){
		return $this->colors['end'];
	}


	/**
	 * Function: _return
	 *
	 * Der Übergebene Text wird zurückgegeben, dabei kann eine Mindestlänge des Textes angegeben werden.
	 * Ist der Text kürzer als die übergebene Mindestlänge, wird der Rest mit Leerzeichen aufgefüllt, bis die Mindestlänge erreicht wurde.
	 *
	 * @param string $text  Der Text, welcher zureückgegeben werden soll
	 * @param int $len      Die Mindestlänge des Textes
	 * @return string       Der Text mit der Mindestlänge
	 * @since 1.0
	 *
	 */
	function _return($text, $len=0){
		$ret = $text;
		if(strlen($ret) < $len){
			for ($i=strlen($ret); $i < $len; $i++) {
				$ret.=" ";
			}
		}
		return $ret;
	}

	/**
	 * Function: _echo
	 *
	 * Gibt auf der CLI einen buten Text aus, und hängt automatisch einen Zeilenumbruch an die Ausgabe.
	 *
	 * @param  string $text   Der auszugebene Text
	 * @param  string $color  Die zu verwendente Farbe
	 * @param  bool $newline  Soll am ende der Zeile ein \n ausgegeben werden (Default: true)
	 * @return void
	 * @since 1.0
	 * @version 1.7.4
	 *
	 */
	function _echo($text, $color='light_green', $newline = true){
		echo $this->startColor($color).$text.$this->endColor();
		if($newline){
			echo PHP_EOL;
		}
	}
}
