<?php
/**
 * phpNSTA
 *
 * This file is part of the phpNSTA
 *
 * @license http://www.gnu.de/documents/gpl-2.0.de.html
 * @copyright 2013 - 2016
 */

/**
 * XMLRPC API Handler
 *
 * Übernimmt die Kommunikation mit einer XMLRPC fähigen API
 * und parst das Ergebnis
 *
 * @package phpNSTA
 * @author Michael Ziegler <michael.ziegler@it-novum.com>
 * @author Daniel Ziegler <daniel.ziegler@it-novum.com>
 */
class XMLRPCProxy{
	/** @var string Die URL der XMLRPC API, welche abgefragt werden soll */
	public $url;
	
	/** @var string Bereich der XMLRPC API */
	public $namespace;

	/**
	 * Function: XMLRPCProxy == __construct
	 *
	 * Erstellt das Object Supervisor
	 *
	 * @param string $url        Die URL der XMLRPC API
	 * @param string $namespace  Der Bereicht der API, welcher abgefragt werden soll (Default: "")
	 * @return object            Supervisor XMLRPC API als Object
	 * @since 1.4
	 *
	 */
	public function XMLRPCProxy($url, $namespace=""){
		$this->url = $url;
		$this->namespace = $namespace;
	}

	/**
	 * Function: do_call
	 *
	 * Sendet an Aufruf an die XML-RPC API von Supervisor
	 *
	 * @param string $function_name Der Name der Methode, die von der XMLRPC API angesprochen werden soll
	 * @param mixed $function_args  Die Argumente, die an die Methode übergeben werden sollen
	 * @return mixed    Den Rückgabewert der Supervisor XML-RPC API
	 * @since 1.4
	 *
	 */
	public function do_call($function_name, $function_args){
		$request = xmlrpc_encode_request($function_name, $function_args);
		$header[] = "Content-type: text/xml";
		$header[] = "Content-length: ".strlen($request);
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $this->url);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_TIMEOUT, 10);
		curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $request);
                curl_setopt($ch, CURLOPT_HTTPPROXYTUNNEL, false);
		curl_setopt($ch, CURLOPT_PROXY, '');

		$data = curl_exec($ch);
		if(curl_errno($ch)){
			return curl_error($ch);
		}else{
			curl_close($ch);
			return xmlrpc_decode($data);
		}
	}

	/**
	 * Function: __get
	 *
	 * Parst den Aufruf des XMLRPCProxy Objectes und führ die Gewünschte Operation aus
	 *
	 * @param string $key  Der abzufragende Wert
	 * @return object      Ein neues verschachtelte XMLRPCProxy Object
	 * @since 1.4
	 *
	 */
	public function __get($key){
		if($this->namespace !== ""){
			return new XMLRPCProxy($this->url, "{$this->namespace}.$key");
		}else{
			return new XMLRPCProxy($this->url, $key);
		}
	}
	
	/**
	 * Function: __call
	 *
	 * Führt den Aufruf aus
	 *
	 * @param string $func Der Name der Methode, die von der XMLRPC API angesprochen werden soll
	 * @param mixed $args  Die Argumente, die an die Methode übergeben werden sollen
	 * @return Mixed 	Den Rückgabewert der Supervisor XML-RPC API
	 * @since 1.4
	 *
	 */	
	public function __call($func, $args){
		if($this->namespace !== ""){
			return $this->do_call("{$this->namespace}.$func", $args);
		}else{
			return $this->do_call($func, $args);
		}
	}
}

