<?php
/**
 * phpNSTA
 *
 * This file is part of the phpNSTA
 *
 * @license http://www.gnu.de/documents/gpl-2.0.de.html
 * @copyright 2013 - 2016
 */

/**
 * MySQL Klasse
 *
 * Über diese Klasse kann eine Verbindung zu einer MySQL Datenbank hergestellt werden
 * und MySQL Abfragen (select * from *) ausgeführt werden
 *
 * @package phpNSTA
 */
class MySQLHandler{
	/** @var string IP-Addresse oder Hostname des MySQL Servers */
	private $SERVER = '';

	/** @var int|string Port des MySQL Servers */
	private $PORT = '';

	/** @var string Name der Datenbank */
	private $DATENBANK = '';

	/** @var string MySQL Benutzername */
	private $BENUTZER = '';

	/** @var string MySQL Passwort */
	private $PASSWORT = '';

	/** @var resource MySQL Verbindung als Ressource */
	private $CONNECTION;

	/** @var bool Ob eine offene MySQL Transaktion vorliegt */
	private $openTransaction = false;

	/** @var bool Sollen Fehler in der Datenbank loggt werden */
	private $logging = true;


	/** @var phpVersion  */
	private $phpVersion;

	/**
	 * Konstruktor, der eine Verbindung zur Datenbank herstellt
	 *
	 * @param string $server     IP-Adresse (oder Servername) des DB-Servers
	 * @param string|int $port   Port, auf dem die Datenbank laeuft
	 * @param string $datenbank  Name der Datenbank
	 * @param string $benutzer   Benutzername zur Anmeldung an der Datenbank
	 * @param string $passwort   Zum Benutzername gehoeriges Passwort
	 * @param bool $dieOnError   (optional) Abbruch bei Fehler (default true)
	 */
	function __construct($server, $port, $datenbank, $benutzer, $passwort, $dieOnError = true) {
		$this->SERVER = $server;
		$this->PORT = $port;
		$this->DATENBANK = $datenbank;
		$this->BENUTZER = $benutzer;
		$this->PASSWORT = $passwort;

		$this->phpVersion = new phpVersion();

		if (!$this->_connect() && $dieOnError)
			die("Fehler beim Verbindungsaufbau zur Datenbank '".$server.":".$port."/".$datenbank."'! - Skript gestoppt!");
	}

	/**
	 * Stellt eine Verbindung zur Datenbank her
	 *
	 * @access private
	 * @return bool
	 */
	private function _connect() {
		if($this->phpVersion->isPhp5()){
                     $this->CONNECTION = @mysql_connect($this->SERVER . ":" . $this->PORT, $this->BENUTZER, $this->PASSWORT, true);
                     if ((!$this->CONNECTION) || (!@mysql_select_db($this->DATENBANK, $this->CONNECTION)))
                            return false;
                     mysql_query("SET NAMES 'utf8'", $this->CONNECTION);
                    return true;
		}

		$this->CONNECTION = mysqli_connect($this->SERVER, $this->BENUTZER, $this->PASSWORT, $this->DATENBANK, $this->PORT);
		if ((!$this->CONNECTION) || (!@mysqli_select_db($this->CONNECTION, $this->DATENBANK)))
			return false;
		mysqli_query($this->CONNECTION, "SET NAMES 'utf8'");
		return true;
	}

	/**
	 * Prueft den Verbindungsstatus der Datenbank
	 *
	 * @return bool Status
	 */
	public function checkConnection() {
                if($this->phpVersion->isPhp5()){
		    if (!$this->CONNECTION || !@mysql_select_db($this->DATENBANK, $this->CONNECTION))
		        return false;
		    return true;
                }

                if (!$this->CONNECTION || !@mysqli_select_db($this->CONNECTION, $this->DATENBANK))
                    return false;
                return true;
	}


	/**
	 * Maskiert eine Zeichenkette zur Verwendung in SQL-Statements
	 *
	 * @param string $str zu maskierende Zeichenkette
	 * @return string maskierte Zeichenkette
	 */
	public function escStr($str) {
            if($this->phpVersion->isPhp5()){
		return mysql_real_escape_string($str, $this->CONNECTION);
            }
            return mysqli_real_escape_string($this->CONNECTION, $str);
	}

	/**
	 * Liefert die Datenbankverbindung.
	 *
	 * @return resource die Datenbankverbindung
	 */
	public function getConnection() {
		return $this->CONNECTION;
	}


	/**
	 * Sendet ein SELECT-Statement an die Datenbank und liefert das Ergebnis als mehrdimensionales Array zurueck
	 *
	 * @param string $sql SQL-Befehl ("SELECT ...")
	 * @return array Array, das in der ersten Dimension die Zeilen (Schluessel ist Zeilennummer (0-basierend)) und in der zweiten Dimension die Spalten (Schluessel ist sowohl Spaltennummer (0-basierend) als auch Spaltename) enthaelt
	 */
	public function select($sql) {
		if($this->phpVersion->isPhp5()){
		    if(!mysql_select_db($this->DATENBANK, $this->CONNECTION))
		        $this->showError("Fehler beim Wechseln der Datenbank");
		    $results = mysql_query($sql, $this->CONNECTION);
		    if (!$results)
		        $this->showError(mysql_error($this->CONNECTION).": ".$sql);
		    $data = array ();
		    while ($row = mysql_fetch_array($results))
		        $data[] = $row;
		    mysql_free_result($results);
		    return $data;
		}

                if(!mysqli_select_db($this->CONNECTION, $this->DATENBANK))
                    $this->showError("Fehler beim Wechseln der Datenbank");
		$results = mysqli_query($this->CONNECTION, $sql);
		if (!$results)
		    $this->showError(mysqli_error($this->CONNECTION).": ".$sql);
		$data = array ();
		while ($row = mysqli_fetch_array($results))
		    $data[] = $row;
		mysqli_free_result($results);
		return $data;
	}

	/**
	 * Schaltet das Logging ein (true) oder aus (false)
	 * @param bool $logging Schalter fuer Logging
	 */
	public function setLogging($logging) {
		$this->logging = $logging;
	}

	/**
	 * Gibt eine Fehlermeldung aus (und loggt diese in die Datenbank, wenn Logging aktiviert ist)
	 * @access private
	 * @param string $errStr Fehlermeldung
	 */
	private function showError($errStr) {
		if($this->logging) {
			$this->logDBError("DB", "MySQLHandler", $errStr);
		}
		die("ERROR: ".$errStr);
	}

	/**
	 * Speichert eine Fehlermeldung in der Datenbank ab
	 * @param prozess         "Prozess", in dem der Fehler aufgetreten ist (frei)
	 * @param funktion        Funktion, in der der Fehler aufgetreten ist
	 * @param error           Fehlermeldung
	 * @param rekursionslevel Rekursionslevel, der dazu dient, bei rekursiven Aufrufen Endlosschleife zu vermeiden (optional, Standard: 0)
	 * @global object $DB     Datenbank-Object selbst
	 * @return Liefert die uebergebene Fehlermeldung wieder zurueck
	 */
	function logDBError($prozess, $funktion, $error, $rekursionslvl = 0) {
		//This is not supported in openITCOCKPIT 3
		return $error;
	}
}
/**
 * MySQL Exception
 *
 * Leere Klasse, welche im Moment noch nicht benutzt wird.
 *
 * @package phpNSTA
 */
class MySQLException extends Exception
{

}
