#!/usr/bin/env php
<?php
/**
 * phpNSTA
 *
 * This file is part of the phpNSTA
 *
 * @license http://www.gnu.de/documents/gpl-2.0.de.html
 * @copyright 2013 - 2016
 */


require_once "core.php";
Boot::bootup(); //<-- Startup sequence
$CLI = new CLI();
$pid = array();
TSYNC::checkTime();

$ForkInformation = SAT::buildGearmanServerString();

if(Config::val('SSH', 'use_ssh_tunnel') === true){
	//Für jeden Satelliten muss ein SSH Tunnel aufgebaut werden
	$Supervisor = new Supervisor();
	$Supervisor->stop();
	$Supervisor->writeConfiguration(SSH::$sat_systems);
	$Supervisor->restart();
	$Supervisor->start();
	$Supervisor->sleep();
}

/****************** SAT-COM CHILD ******************/
$sat_com_pid = pcntl_fork();
	if(!$sat_com_pid){
		Logger::clog('Forked new satellite communication child', 1, false);
		//declare(ticks=100);
		pcntl_signal(SIGTERM, "sig_handler");
		pcntl_signal(EXPORT_STARTED, "sig_handler");
		pcntl_signal(EXPORT_FINISHED, "sig_handler");
		while(true){
			pcntl_signal_dispatch();
			$SATs_and_Tasks = SAT::checkCMDFiles();
			if(is_array($SATs_and_Tasks) && !empty($SATs_and_Tasks)){
				foreach($SATs_and_Tasks as $SATAsArray){
					$GMAN = new GearmanClient();
					$GMAN->setTimeout(Config::val('GEARMAN', 'timeout'));
					if(Config::val('SSH', 'use_ssh_tunnel') === true){
						//Lokalen Port verwenden
						$GMAN->addServer($SATAsArray['local_ip'], $SATAsArray['local_port']);
					}else{
						$GMAN->addServer($SATAsArray['ip'], Config::val('GEARMAN', 'port'));
					}
					if(!$GMAN->ping('Ping')){
						if(Config::val('SSH', 'use_ssh_tunnel') === true){
							Logger::clog('SAT-COM: Gearman Job Server "'.$SATAsArray['local_ip'].'" not reachable', 3, false);
						}else{
							Logger::clog('SAT-COM: Gearman Job Server "'.$SATAsArray['ip'].'" not reachable', 3, false);
						}
						continue;
					}else{
						foreach($SATAsArray['commands'] as $command){
							$GMAN->doBackground('NAGIOS_COMMAND', $command);
						}
					}
					unset($GMAN);
				}
				unset($SATs_and_Tasks);
			}
			sleep(Config::val('SAT', 'interval'));
		}
	}

/****************** SAT CHILD ******************/

$Childs = array();

for($i = 0; ($i + 1) <= $ForkInformation['number_of_workers']; $i++){
	$pid[$i] = pcntl_fork();
	if(Config::val('SSH', 'use_ssh_tunnel') === true){
		$Childs[$pid[$i]] = $ForkInformation[$i];
	}
	if(!$pid[$i]){
		//Ab hier laufen nur noch Kindprozesse (SAT-System Worker)
		createChild($ForkInformation, $i);
	}
}
/****************** MAIN phpNSTA process ******************/
$API = new API();
$API->addPid($pid);
$API->addPid($sat_com_pid);
Logger::log('Starting API-Listener. Parent process stand by.', 1);
$API->addChilds($Childs);
//Ab hier verfält der Parent-Prozess in ein while(true) und wartet auf externe commands und prüft ob alle Verbindungen funktionieren
$API->listener();


/****************** Helper Functions ******************/

/**
 * Function: sig_handler
 *
 * Wird von php automatisch aufgerufen, soblad ein Signal an das Child gesendet wurde
 * zum beispiel mit posix_kill($pid, $signo)
 *
 * @param  int $signo  signal number
 * @return void
 * @author Daniel Ziegler <daniel.ziegler@it-novum.com>
 * @since 1.0
 * @version 1.4.3
 *
 */
function sig_handler($signo){
	Logger::clog('Received signal: '.$signo, 2);
	switch ($signo){
		case SIGTERM: // 15
			Nagios::processedCache(true);
			exit(0);
			break;

		case EXPORT_STARTED: // 1
			Nagios::$export_running = true;
			break;

		case EXPORT_FINISHED: // 2
			Nagios::$export_running = false;
			break;

		case SIGUSR1: // 10
			Nagios::$nagios_runtime_ok = false;
			break;
			
		case SIGUSR2: // 12
			Nagios::$nagios_runtime_ok = true;
			break;

		default:
			Logger::clog('SIGTERM-Event not supported yet!', 0);
			break;
	}
	pcntl_signal(SIGTERM, "sig_handler");
	pcntl_signal(EXPORT_STARTED, "sig_handler");
	pcntl_signal(EXPORT_FINISHED, "sig_handler");
	pcntl_signal(SIGUSR1, "sig_handler");
	pcntl_signal(SIGUSR2, "sig_handler");
}

/**
 * Function: ParseNagiosService
 *
 * Überbit die service check results an Nagios
 *
 * @param object $job  Gearman $job Object
 * @return bool
 * @link http://php.net/manual/de/gearmanworker.addfunction.php
 * @author Daniel Ziegler <daniel.ziegler@it-novum.com>
 * @since 1.0
 *
 */
function ParseNagiosService($job){
	Nagios::Cache(array(unserialize($job->workload())));
	return true;
}

/**
 * Function: ParseNagiosHost
 *
 * Überbit die host check results an Nagios
 *
 * @param object $job Gearman $job Object
 * @return void
 * @author Daniel Ziegler <daniel.ziegler@it-novum.com>
 * @since 1.0
 *
 */
function ParseNagiosHost($job){
	Nagios::Cache(array(unserialize($job->workload())));
	return true;
}

/**
 * Function: doEvents
 *
 * Sorgt dafür, das das alte des lokalen Caches überprüft wird
 * Diese Methode ist mit GEARMAN_WORKER_NON_BLOCKING veraltet!
 *
 * @deprecated Durch GEARMAN_WORKER_NON_BLOCKING veraltet und sollte entfernt werden!
 * @return bool
 * @author Daniel Ziegler <daniel.ziegler@it-novum.com>
 * @since 1.4.3
 *
 */
function doEvents(){
	Logger::clog("Received event from satellite", 6);
	pcntl_signal_dispatch();
	Nagios::valCacheAge();
	return true;
}

/**
 * Function: ParseStatusengineOchp
 *
 * Die Daten werden nicht über das Klassische OCHP geliefert, sondern
 * von Statusengine enable_ochp=1 bereitgestellt
 * Übergibt die host check results an Nagios
 *
 * @param object $job Gearman $job Object
 * @return void
 * @author Daniel Ziegler <daniel.ziegler@it-novum.com>
 * @since 1.7.6
 *
 */
function ParseStatusengineOchp($job){
	$workload = json_decode($job->workload());
	//Logger::dump($workload);
	
	$data = array(
		'TYPE' => 'HOST',
		'TIMET' => $workload->hostcheck->start_time,
		'HOSTNAME' => $workload->hostcheck->host_name,
		'HOSTSTATEID' => $workload->hostcheck->state,
		'HOSTOUTPUT' => $workload->hostcheck->output,
		
		//Mod_Gearman additional fields
		'start_time' => $workload->hostcheck->start_time,
		'end_time' => $workload->hostcheck->end_time,
		'latency' => $workload->hostcheck->latency,
	);
	
	unset($workload);
	Nagios::Cache(array($data));
}

/**
 * Function: ParseStatusengineOcsp
 *
 * Die Daten werden nicht über das Klassische OCSP geliefert, sondern
 * von Statusengine enable_ocsp=1 bereitgestellt
 * Übergibt die service check results an Nagios
 *
 * @param object $job Gearman $job Object
 * @return void
 * @author Daniel Ziegler <daniel.ziegler@it-novum.com>
 * @since 1.7.6
 *
 */
function ParseStatusengineOcsp($job){
	$workload = json_decode($job->workload());
	
	$data = array(
		'TYPE' => 'SERVICE',
		'TIMET' => $workload->servicecheck->start_time,
		'HOSTNAME' => $workload->servicecheck->host_name,
		'SERVICEDESC' => $workload->servicecheck->service_description,
		'SERVICESTATEID' => $workload->servicecheck->state,
		'SERVICEOUTPUT' => $workload->servicecheck->output,
		'SERVICEPERFDATA' => $workload->servicecheck->perf_data,
		'LONGSERVICEOUTPUT' => $workload->servicecheck->long_output,
	
		//Mod_Gearman additional fields
		'start_time' => $workload->servicecheck->start_time,
		'end_time' => $workload->servicecheck->end_time,
		'latency' => $workload->servicecheck->latency,
	);
	
	unset($workload);
	Nagios::Cache(array($data));
}

function ProcessBulkHostRecords($job){
	$data = unserialize($job->workload());
	if($data['compression']){
		if($data['compression'] == 'bz2'){
			Logger::clog('Receive host bulk transmission compressed with bz2', 10);
			$payload = bzdecompress($data['data']);
		}
		
		if($data['compression'] == 'gz'){
			Logger::clog('Receive host bulk transmission compressed with gz', 10);
			$payload = gzuncompress($data['data']);
		}
	}else{
		$payload = $data['data'];
	}
	$records = unserialize($payload);
	Logger::clog('Received '.sizeof($records).' host status records', 10);
	foreach($records as $record){
		Nagios::Cache(array($record));
	}
}

function ProcessBulkServiceRecords($job){
	$data = unserialize($job->workload());
	if($data['compression']){
		if($data['compression'] == 'bz2'){
			Logger::clog('Receive service bulk transmission compressed with bz2', 10);
			$payload = bzdecompress($data['data']);
		}
		
		if($data['compression'] == 'gz'){
			Logger::clog('Receive service bulk transmission compressed with gz', 10);
			$payload = gzuncompress($data['data']);
		}
	}else{
		Logger::clog('Receive uncompressed bulk transmission', 10);
		$payload = $data['data'];
	}
	$records = unserialize($payload);
	Logger::clog('Received '.sizeof($records).' service status records', 10);
	foreach($records as $record){
		Nagios::Cache(array($record));
	}
}

function ProcessCustomRecords($job){
	$input = json_decode($job->workload());
	Logger::clog('Received '.$input->count.' custom data records', 11);
	$GearmanClient = new GearmanClient();
	$GearmanClient->setTimeout(Config::val('GEARMAN', 'timeout'));
	$GearmanClient->addServer('127.0.0.1', 4730);
	foreach($input->data as $row){
		$data = base64_decode($row);
		$GearmanClient->doBackground('phpnsta_custom_data', $data);
	}
	unset($GearmanClient);
}


/**
 * Function: createChild
 *
 * Erstellt einen neuen kind Prozess
 * Dies ist eine Endlosschleife, und stellt den eingeltichen Kindprozess da,
 * welcher die Daten der SAT-Systeme verarbeitet.
 *
 * @param  array $ForkInformation  Array mit allen benötigten Parametern eines SAT-Systems
 * @param  int $i   Array Key, welcher gerade bearbeitet werden soll
 * @return void
 * @author Daniel Ziegler <daniel.ziegler@it-novum.com>
 * @since 1.0
 * @version 1.7.0
 *
 */
function createChild($ForkInformation, $i){
	Logger::$ips = $ForkInformation[$i]['server_string'];
	Logger::$names = $ForkInformation[$i]['server_names'];
	Logger::clog('Forked new GearmanWorker child', 1);
	Nagios::_construct();
	//declare(ticks=100);
	pcntl_signal(SIGTERM, "sig_handler");
	pcntl_signal(EXPORT_STARTED, "sig_handler");
	pcntl_signal(EXPORT_FINISHED, "sig_handler");
	pcntl_signal(SIGUSR1, "sig_handler");
	pcntl_signal(SIGUSR2, "sig_handler");
	
	$use_multiple_worker = Config::val('GEARMAN', 'use_multiple_worker');
	
	if(!$use_multiple_worker){
		$worker = new GearmanWorker();
	
		//New since 1.7.0
		$worker->addOptions(GEARMAN_WORKER_NON_BLOCKING);
	
	
		$worker->setTimeout(Config::val('GEARMAN', 'timeout'));
		Logger::clog('My Gearman Job Servers: '.$ForkInformation[$i]['server_string'], 2);
		$worker->addServers($ForkInformation[$i]['server_string']);

		if(Config::val('GEARMAN', 'disable_legacy_queues') === false){
			$worker->addFunction("PARSE_SERVICE", "ParseNagiosService");
			$worker->addFunction("PARSE_HOST", "ParseNagiosHost");
			$worker->addFunction("DO_EVENTS", "doEvents");
		}
		
		if(Config::val('GEARMAN', 'bulk_transmit') === true){
			$worker->addFunction("ochp_bulk", "ProcessBulkHostRecords");
			$worker->addFunction("ocsp_bulk", "ProcessBulkServiceRecords");
			$worker->addFunction("phpnsta_custom_bulk", "ProcessCustomRecords");
		}else{
			//Add Statusengine queues
			$worker->addFunction("statusngin_ochp", "ParseStatusengineOchp");
			$worker->addFunction("statusngin_ocsp", "ParseStatusengineOcsp");
		}
		
		if(!$worker->echo('Ping')){
			Logger::clog('Gearman Job Server "'.$ForkInformation[$i]['server_string'].'" not reachable', 3);
		}
		
	}else{
		$workers = array();
	
		foreach($ForkInformation[$i]['servers'] as $server){
			$worker = new GearmanWorker();
	
			//New since 1.7.0
			$worker->addOptions(GEARMAN_WORKER_NON_BLOCKING);
	
	
			$worker->setTimeout(Config::val('GEARMAN', 'timeout'));
			Logger::clog('Create dedicated GearmanWorker for: '.$server, 2);
			$AddressAndPort = explode(':', $server);
			$worker->addServer($AddressAndPort[0], $AddressAndPort[1]);
			//$worker->addServers("172.16.2.13");
			
			if(Config::val('GEARMAN', 'disable_legacy_queues') === false){
				$worker->addFunction("PARSE_SERVICE", "ParseNagiosService");
				$worker->addFunction("PARSE_HOST", "ParseNagiosHost");
				$worker->addFunction("DO_EVENTS", "doEvents");
			}
		
			if(!$worker->echo('Ping')){
				Logger::clog('Gearman Job Server "'.$server.'" not reachable', 3);
			}
		
			$workers[] = $worker;
			unset($worker, $AddressAndPort);
		}
	}

	$maxidle = (int)Config::val('CPU', 'maxidle');
	$jobIdleCounter = 0;

	/*
	while(@$worker->work()							||
		  $worker->returnCode() == GEARMAN_IO_WAIT	|| // 1
		  $worker->returnCode() == GEARMAN_NO_JOBS	|| // 35
		  true){
	*/
	while(true){
		pcntl_signal_dispatch();
		
		if(Nagios::$nagios_runtime_ok === true){
			if($use_multiple_worker){
				$sleep = array();
				foreach($workers as $worker){
					$workerResult = $worker->work();
					$rc = $worker->returnCode();
					if($rc == GEARMAN_NO_JOBS || $rc == GEARMAN_IO_WAIT){
						if($jobIdleCounter < $maxidle){
							$jobIdleCounter++;
						}
					}else{
						$jobIdleCounter = 0;
					}
					/*if($workerResult === false){
						//Worker returend false, looks like the queue is empty
						if($jobIdleCounter < $maxidle){
							$jobIdleCounter++;
						}
					}else{
						//We get a job, set $jobIdleCounter back to 0
						$jobIdleCounter = 0;
					}*/
					
				}
				if($jobIdleCounter === $maxidle){
					//Logger::clog('Will start usleep', 7);
					Nagios::valCacheAge(true);
					usleep(250000);
				}
			}else{
				$worker->work();
				
				if($worker->returnCode() == GEARMAN_UNEXPECTED_PACKET){
					//New since 1.7.0
					Logger::clog('Error: GEARMAN_UNEXPECTED_PACKET create new GearmanWorker', 2);
					unset($worker);
					sleep(1);
					
					$worker = new GearmanWorker();
	
					$worker->addOptions(GEARMAN_WORKER_NON_BLOCKING);
	
	
					$worker->setTimeout(Config::val('GEARMAN', 'timeout'));
					Logger::clog('My Gearman Job Servers: '.$ForkInformation[$i]['server_string'], 2);
					$worker->addServers($ForkInformation[$i]['server_string']);
					//$worker->addServers("172.16.2.13");
					
					if(Config::val('GEARMAN', 'disable_legacy_queues') === false){
						$worker->addFunction("PARSE_SERVICE", "ParseNagiosService");
						$worker->addFunction("PARSE_HOST", "ParseNagiosHost");
						$worker->addFunction("DO_EVENTS", "doEvents");
					}
		
					if(!$worker->echo('Ping')){
						Logger::clog('Gearman Job Server "'.$ForkInformation[$i]['server_string'].'" not reachable', 3);
					}
					
				}
				
				Nagios::valCacheAge(true);

				if($worker->returnCode() == GEARMAN_SUCCESS){
					continue;
				}
				if(!@$worker->wait()){
					if($worker->returnCode() == GEARMAN_NO_ACTIVE_FDS){
						//Lost connection - lets wait a bit
						sleep(1);
					}
				}
				
			}
		}
		
		
		if(Nagios::$nagios_runtime_ok === false){
			sleep(1);
		}
	}
}

