<?php
/**
 * phpNSTA
 *
 * This file is part of the phpNSTA
 *
 * @license http://www.gnu.de/documents/gpl-2.0.de.html
 * @copyright 2013 - 2016
 */

/**
 * Verwaltet die Konfigurationsdatei des phpNSTA
 *
 * Diese Klasse exiteirt sowohl für den phpNSTA als auch für den phpNSTAClient.
 * Sie übernimmt die Verwaltung der Konfigurationsdatei config.php.
 * Aus der Konfiguration kann nur gelesen werden.
 * Der Zugriff erfolgt aus­schließ­lich über die statische Methode Config::val();
 *
 *
 * @usedby API
 * @usedby CLI
 * @usedby Supervisor
 * @usedby Boot
 * @usedby Crypt
 * @usedby Logger
 * @usedby Nagios
 * @usedby SAT
 * @usedby SSH
 * @usedby TSYNC
 * @package phpNSTA
 * @author Daniel Ziegler <daniel.ziegler@it-novum.com>
 */
class Config{
	
	/** @var array Array der Konfigurationsdatei als Cache */
	protected static $config = array();
	
	/**
	 * Function: val
	 *
	 * Liefert einen Wert aus der Datei config.php zurück
	 *
	 * @param  string $MainKey  Der Hauptkey, des Konfigurationsarray (z. B. NAGIOS oder MAIN)
	 * @param  string $SubKey   Der Unterkey des Konfigurationsarray (z. B. cmd_file)
	 * @return mixed            Der gefundene Wert oder false, wenn kein Wert gefunden wurde
	 * @since 1.0
	 *
	 */
	public static function val($MainKey = null, $SubKey = null){
		if(empty(self::$config) || !is_array(self::$config)){
			require_once "config.php";
			self::$config = $config;
		}
		
		if(isset(self::$config[$MainKey][$SubKey])){
			return self::$config[$MainKey][$SubKey];
		}
		
		Logger::log('Config key "'.$MainKey.'", "'.$SubKey.'" not found in configuration file! (config.php)', 2);
		return false;
	}
}
