<?php
/**
 * phpNSTA
 *
 * This file is part of the phpNSTA
 *
 * @license http://www.gnu.de/documents/gpl-2.0.de.html
 * @copyright 2013 - 2016
 */

/**
 * Statische Klasse für Loggin
 *
 * Diese Klasse ermöglicht es sowohl dem Haupt Prozess, als auch allen Kindprozessen
 * in das Logfile des phpNSTA /var/log/phpNSTA.log zu schreiben.
 * Zusätzlich kann das Logfile rotiert werden 
 *
 *
 * @uses Config - Statische Klasse um die Konfigurationsdatei zu lesen
 * @usedby *
 * @package phpNSTA
 * @author Daniel Ziegler <daniel.ziegler@it-novum.com>
 */
class Logger{
	
	/** @var resource Das Logfile als Ressource */
	public static $logfile = null;
	
	/**
	 * @var int Maximaler loglevel, wird nicht verwendet
	 * @deprecated Wird nicht verwendet, sollte entfernt werden
	 */
	public static $valid_until = 15;
	
	/** @var bool Ob "heute" ein Logrotate schon durchgeführt wurde */
	public static $done_logratate = false;
	
	/** @var int Wird verwendet um zu prüfen, ob das logrotate schon duchgeführt wurde */
	public static $logrotate_freshness = 4000;
	
	/** @var int Timestamp wann das letzte logrotate durchgeführt wurde */
	public static $done_logrotate_on = 0;
	
	/** @var string IP-Addressen (der SAT-Systeme) des Kindprozesses, welcher gerade in das Logfile schreibt */
	public static $ips = "";
	
	/** @var string Name (der SAT-Systeme) des Kindprozesses, welcher gerade in das Logfile schreibt */
	public static $names = "";
	
	/**
	 * Function: log
	 *
	 * Schreibt einen beliebigen Test in das Logfile
	 * Wrapper-Funktion für self::write()
	 * 
	 *
	 * @param  string $str  Der Text für das Logfile (Default: '')
	 * @param  int $loglevel  Loglevel als integer (Default:0)
	 * @return void
	 * @since 1.0
	 *
	 */
	public static function log($str = '', $loglevel = 0){
		self::write($str, $loglevel);
	}
	
	/**
	 * Function: clog
	 *
	 * Schreibt einen beliebigen Test in das Logfile
	 * Sollte immer von Child Prozessen verwendet werden!
	 * Wrapper-Funktion für self::write()
	 * 
	 *
	 * @param  string $str  Der Text für das Logfile (Default: '')
	 * @param  int $loglevel  Loglevel (Default: 0)
	 * @param  bool $show_name  Ob der Name der Name der SAT-Systeme mit angezeigt werden soll (Default: true)
	 * @return void
	 * @since 1.0
	 * @version 1.1
	 *
	 */

	public static function clog($str = '', $loglevel = 0, $show_name=true){
		$name = "";
		if(Config::val('LOG', 'full_names') && $show_name){
			$name = " -> ".self::$names." (".self::$ips.")";
		}
		self::write("    Child [".getmypid().$name."]: ".$str, $loglevel);
	}
	
	/**
	 * Function: write
	 *
	 * Schreibt einen beliebigen Test in das Logfile
	 * 
	 *
	 * @param  string $str  Der Text für das Logfile (Default: '')
	 * @param  int $loglevel  Loglevel (Default: 0)
	 * @return void
	 * @since 1.0
	 *
	 */

	public static function write($str = '', $loglevel = 0){
		if(!is_resource(self::$logfile)){
			self::open();
		}
		
		if($loglevel <= Config::val('LOG', 'loglevel')){
			fwrite(self::$logfile, date(Config::val('LOG', 'date_format'))." => ".$str."\n");
		}
	}
	
	/**
	 * Function: dump
	 *
	 * Schreibt den kompletten Inhalt und Aufbau einer Variable in das Logfile,
	 * vergleichbar mit print_r()
	 *
	 * @param  mixed $a Die zu dumpende Variable
	 * @return void
	 * @since 1.0
	 *
	 */
	public static function dump($a=array()){
		if(!is_resource(self::$logfile)){
			self::open();
		}
		
		fwrite(self::$logfile, var_export($a, true));
	}
	
	/**
	 * Function: open
	 *
	 * Öffnet das Logfile und speichert die Ressource in self::$logfile
	 *
	 * @return void
	 * @since 1.0
	 *
	 */
	public static function open(){
		self::$logfile = fopen(Config::val('LOG', 'logfile'), 'a+');
	}
	
	/**
	 * Function: close
	 *
	 * Schließt das Logfile
	 *
	 * @return void
	 * @since 1.0
	 *
	 */
	public static function close(){
		if(is_resource(self::$logfile)){
			fclose(self::$logfile);
		}
	}
	
	/**
	 * Function: welcome
	 *
	 * Schreibt die Willkommensnachricht des phpNSTA in das Logfile
	 *
	 * @return void
	 * @since 1.0
	 *
	 */
	public static function welcome(){
		self::log("
				|-------------------------------------------------------------------|
				|                      _           _   _ ____ _____  _              |
				|                _ __ | |__  _ __ | \ | / ___|_   _|/ \             |
				|               | '_ \| '_ \| '_ \|  \| \___ \ | | / _ \            |
				|               | |_) | | | | |_) | |\  |___) || |/ ___ \           |
				|               | .__/|_| |_| .__/|_| \_|____/ |_/_/   \_\          |
				|               |_|         |_|                                     |
				|                                                                   |
				| * phpNSTA ".self::_return(Config::val('MAIN', 'version'), 6)."                                                  |
				|                                                                   |
				| * Developed by Daniel Ziegler 2012-".date('Y')." - it-novum GmbH           |
				| * www.it-novum.com                                                |
				| * www.open-itcockpit.org                                          |
				|-------------------------------------------------------------------|
				\n", 0);
	}
	
	/**
	 * Function: _return
	 *
	 * Der Übergebene Text wird zurückgegeben, dabei kann eine Mindestlänge des Textes angegeben werden.
	 * Ist der Text kürzer als die übergebene Mindestlänge, wird der Rest mit Leerzeichen aufgefüllt, bis die Mindestlänge erreicht wurde.
	 *
	 * @param string $text  Der Text, welcher zureückgegeben werden soll
	 * @param int $len      Die Mindestlänge des Textes
	 * @return string       Der Text mit der Mindestlänge
	 * @since 1.0
	 *
	 */
	public static function _return($text, $len=0){
		$ret = $text;
		if(strlen($ret) < $len){
			for ($i=strlen($ret); $i < $len; $i++) {
				$ret.=" ";
			}
		}
		return $ret;
	}
	
	/**
	 * Function: wipe
	 *
	 * Kürzt das Logfile des phpNSTA auf eine Länge von 0 und setzt die benötigten Dateirechte
	 *
	 * @return void
	 * @since 1.0
	 *
	 */
	public static function wipe(){
		fclose(fopen(Config::val('LOG', 'logfile'), 'w+'));
	}
	
	/**
	 * Function: logrotate
	 *
	 * Führt das Logrotate aus und erstellt eine zip Datei.
	 * Nach erfolgreicher Erstellung der zip Datei wird self::wipe aufgerufen.
	 *
	 * @return bool true wenn erfolgreich, false wenn das zip file für das logrotate nicht erstellt werden kann
	 * @since 1.0
	 * @version 1.3
	 *
	 */
	public static function logrotate(){
		if(Config::val('LOG', 'logrotate') && Config::val('LOG', 'loglevel') >= 0){
			if(date("H:i") == Config::val('LOG', 'logrotate_time') && !self::$done_logratate){
				//Do Logrotate
				
				//Check for Cleanup
				if(Config::val('LOG', 'cleanup_logfile') === true && is_numeric(Config::val('LOG', 'cleanup_fileage'))){
					$old_logfiles = glob(Config::val('LOG', 'logrotate_path').'phpNSTA_*.zip');
					if(!empty($old_logfiles)){
						foreach($old_logfiles as $old_logfile){
							if(filemtime($old_logfile) < (time() - Config::val('LOG', 'cleanup_fileage') * 86400 )){
								unlink($old_logfile);
							}
						}
					}
				}
				self::$done_logrotate_on = time();
				self::$done_logratate = true;
				$zip = new ZipArchive();
				if($zip->open(Config::val('LOG', 'logrotate_path').'phpNSTA_'.date(Config::val('LOG', 'logrotate_date_format')).'.zip', ZIPARCHIVE::CREATE) === true){
					self::log('Starting logrotate...', 0);
					self::close();
					$zip->addFile(Config::val('LOG', 'logfile'), 'phpNSTA.log_'.date(Config::val('LOG', 'logrotate_date_format')));
					$zip->setArchiveComment('This file was created automatically by phpNSTAs logrotate function. Created: '.date(Config::val('LOG', 'logrotate_date_format')));
					$zip->close();
					self::wipe();
					self::open();
					self::welcome();
					self::log('Logrotate done...', 0);
				}else{
					self::log('Canot create zip File for logrotate!', 0);
					return false;
				}
			}
			
			//Prüfen ob der Logroate durchgeführt wurde, und $done_logratate wieder auf false gesetzt werden muss
			if((time() - self::$logrotate_freshness) > self::$done_logrotate_on){
				self::$done_logratate = false;
			}
		}
		return true;
	}
}
