<?php
/**
 * phpNSTA
 *
 * This file is part of the phpNSTA
 *
 * @license http://www.gnu.de/documents/gpl-2.0.de.html
 * @copyright 2013 - 2016
 */

/**
 * Statische Klasse zur Synchronisierung der Uhrzeit mit den SAT-Systemen
 *
 * Diese Klasse übernimmt die Verwaltung der Ports, auf die die Verbindung über SSH getunnelt wird
 *
 * @uses Config - Statische Klasse um die Konfigurationsdatei zu lesen
 * @uses Logger - Statische Klasse um in der phpNSTA.log zu schreiben
 * @uses SSH - Statische Klasse welche die Verwaltung der SSH Verbindungen übernimmt
 * @usedby core.php
 * @usedby main.php
 * @usedby API
 * @usedby SSH
 * @usedby Supervisor
 * @usedby SAT
 * @package phpNSTA
 * @author Daniel Ziegler <daniel.ziegler@it-novum.com>
 */
class TSYNC{
	
	/** @var int  Timestamp wann die letzte Synchronisierung ausgeführt wurde */
	public static $last_sync = 0;
	
	/** @var array  Intervall wie oft die Synchronisierung ausgeführt werden soll */
	public static $times = array(
		'minute' => 60,
		'hour' => 3600,
		'day' => 86400
	);
	
	/**
	 * Function: checkTime
	 *
	 * Prüft ob die Uhrzeit vom Master auf die SAT-Systeme übertragen werden muss.
	 * Wenn ja, wird self::SyncTime() aufgerufen
	 *
	 * @return bool  false
	 * @since 1.4
	 *
	 */
	public static function checkTime(){
		if((time() - self::$last_sync) > self::$times[Config::val('TSYNC', 'every')]){
			self::$last_sync = time();
			self::SyncTime();
		}
		return false;
	}
	
	/**
	 * Function: SyncTime
	 *
	 * Sync die Uhrzeit vom Master auf alle SAT-Systeme
	 * Dafür wird über Gearman eine Verbindung mit jedem SAT-System aufgebaut
	 * und die neue Uhrzeit in die Q SET_SYSTEMTIME geschrieben
	 *
	 * @return void
	 * @since 1.4
	 *
	 */
	public static function SyncTime(){
		if(Config::val('SSH', 'use_ssh_tunnel') === true){
			foreach(SSH::$sat_systems as $satellite){
				$GMAN = new GearmanClient();
				Logger::log("TSYNC Daemon: Setting system clock on ".$satellite['name']);
				$GMAN->setTimeout(Config::val('GEARMAN', 'timeout'));
				$GMAN->addServer($satellite['local_ip'], $satellite['local_port']);
				$GMAN->doBackground('SET_SYSTEMTIME', time());
				unset($GMAN);
			}
		}else{
			foreach(SAT::$satellites as $satellite){
				$GMAN = new GearmanClient();
				Logger::log("TSYNC Daemon: Setting system clock on ".$satellite['name']);
				$GMAN->setTimeout(Config::val('GEARMAN', 'timeout'));
				$GMAN->addServer($satellite['ip'], Config::val('GEARMAN', 'port'));
				$GMAN->doBackground('SET_SYSTEMTIME', time());
				unset($GMAN);
			}	
		}
		self::$last_sync = time();
	}
}
